==> free-sa-1.5.1.tar.gz: free-sa-1.5.1/src/logformats/l_logformats.c <==
#include "global.h"


struct MailRecord *sMR = NULL;

struct LogFormat LogFormats[6] = { 
    {	"Squid 2.x native log format", 
	NULL,
	SquidReadRecord, 
	SquidReadDT,
	SquidReadFullDT,
	CommonReadAlign, {
	    1, 1, 1, 1, 1, 1, 1, /* TT reports */
	    1, 1, 1, 1, 1  	 /* TopSites, User, UserFull, UserGraph, UserDT */
	},
	1,
	PROXY },
	
    {	"CERN/NCSA Common Log Format (CLF)", 
	NULL,
	CLFReadRecord, 
	CLFReadDT,
	CLFReadFullDT,
	CommonReadAlign, {
	    1, 1, 1, 1, 1, 1, 1, /* TT reports */
	    0, 1, 1, 1, 1  	 /* TopSites, User, UserFull, UserGraph, UserDT */
	},
	1,
	WWW },
	
    {	"CERN/NCSA Combined Log Format", 
	NULL,
    	CombinedReadRecord, 
	CLFReadDT,
	CLFReadFullDT,
	CommonReadAlign, {
	    1, 1, 1, 1, 1, 1, 1, /* TT reports */
	    0, 1, 1, 1, 1     	 /* TopSites, User, UserFull, UserGraph, UserDT */
	},
	1,
	WWW },
	
    {	"Postfix 2.x over syslog log format",
	PFSLReadRecordI, 
	MailReadRecord, 
	SyslogReadDT,
	SyslogReadFullDT,
	CommonReadAlign, {
	    0, 0, 1, 1, 0, 0, 1, /* TT reports */
	    1, 1, 1, 1, 1	 /* TopSites, User, UserFull, UserGraph, UserDT */
	},
	1,
	MAIL },
	
    {	"Qmail over syslog log format",
	QMSLReadRecordI,
	MailReadRecord, 
	SyslogReadDT,
	SyslogReadFullDT,
	CommonReadAlign, {
	    0, 0, 1, 1, 0, 0, 1, /* TT reports */
	    1, 1, 1, 1, 1	 /* TopSites, User, UserFull, UserGraph, UserDT */
	},
	1,
	MAIL },
	
    {	"Communigate pro 5.x native log format",
	CGPReadRecordI,
	MailReadRecord, 
	SyslogReadDT,
	SyslogReadFullDT,
	CommonReadAlign, {
	    0, 0, 1, 1, 0, 0, 1, /* TT reports */
	    1, 1, 1, 1, 1	 /* TopSites, User, UserFull, UserGraph, UserDT */
	},
	1,
	MAIL }
};


void CommonReadAlign(FILE *iLog) {
    char t;
    
    while (fread(&t, 1L, 1L, iLog) == 1L) if (t == '\n') return;
    if (ferror(iLog)) SAexit("");
}


void CLFDTZ(time_t *iTime, char *idt) {
    struct tm ttm;
    
    if (strptime(idt, "%d/%b/%Y:%T %z", &ttm) == NULL) SAexit("Error converting date %s", idt);

    *iTime = SAmktime(&ttm);
}


char CLFReadDT(FILE *iLog, time_t *iTime) {
    char tdt[27];
    
    if (fscanf(iLog, "%*[^[]%*c%[^]]%*[^\n]%*c", tdt) != 1) return 0;
    
    CLFDTZ(iTime, tdt);
    
    return 1;
}


char CLFReadFullDT(FILE *iLog, time_t *iTime, char *iBuf) {
    char tdt[27];
    
    if (fgets(iBuf, RECORDSIZE, iLog) == NULL) return 0;
    if (sscanf(iBuf, "%*[^[]%*c%[^]]", tdt) != 1) return 0;
    
    CLFDTZ(iTime, tdt);
    
    return 1;
}


void SyslogDTZ(time_t *iTime, char *idt, short int year) {
    struct tm ttm;
    
    if (strptime(idt, "%b %d %T", &ttm) == NULL) SAexit("Error converting date %s", idt);

    ttm.tm_year = (int)(year - 1900);

    *iTime = SAmktime(&ttm);    
}


char SyslogReadDT(FILE *iLog, time_t *iTime) {
    char tdt[16];

    if (fscanf(iLog, "%15[^/] %*[^\n]%*c", tdt) != 1) return 0;
    
    SyslogDTZ(iTime, tdt, SYSLOGYEAR);
    
    return 1;
}


char SyslogReadFullDT(FILE *iLog, time_t *iTime, char *iBuf) {
    char tdt[16];

    if (fgets(iBuf, RECORDSIZE, iLog) == NULL) return 0;
    if (sscanf(iBuf, "%15[^/]", tdt) != 1) return 0;
    
    SyslogDTZ(iTime, tdt, SYSLOGYEAR);
    
    return 1;
}


char MailReadRecord(FILE *tLog, time_t *tTime, double *tSeconds, char *tIp, short int *tCode, double *tBytes, char *tMethod, char *tUrl, char *tName, char *tStat) {
    struct MailRecord *t = NULL, *nt;

    if ((t = READ_RECORDI(tLog)) == NULL) {
	for (t = sMR; t != NULL; t = nt) {
	    nt = t->next;
	    
	    if (t->status != 3) DelMR(t);
	    else break;
	}
    }
    
    if (t == NULL) return 0;
    
    strcpy(tIp, t->ip);
    strcpy(tUrl, t->url);
    strcpy(tName, t->name);
    *tBytes = t->bytes;
    *tTime = t->time;
    *tCode = t->code;
    *tStat = t->stat;
    *tSeconds = 0;
    tMethod[0] = 'G';

    if (--t->count) t->status = 2;			/* if multirecepient */
    else DelMR(t);					/* if last recepient */

    return 1;
}


struct MailRecord *FindNewMR(unsigned int msgid) {
    struct MailRecord *t = sMR, *ot = NULL;
    
    /* find MR by msgid */
    while(t != NULL) {
	if (t->msgid == msgid) return t; /* MR already exist */
	ot = t;
	t = t->next;
    }
    
    /* init new MR */
    t = (struct MailRecord *)SAmalloc(sizeof(struct MailRecord));
    t->msgid = msgid;
    t->status = -1;
    t->name[0] = '-';			/* handle empty from: <> */
    t->code = 0;
    t->count = 0;
    t->next = NULL;
    
    if (ot != NULL) ot->next = t;	/* init previous MR next */
    else sMR = t;			/* init start MR */
    
    return t;				/* MR not exist */
}


struct MailRecord *FindDupMR(unsigned int msgid) {
    struct MailRecord *t = sMR, *nt;
    
    /* find MR by msgid */
    while (t != NULL) {
	if (t->msgid == msgid) {	/* MR found */
	    if (!t->count) {		/* 1st recepient */
		t->count++;
		return t;
	    }
	    /* duplicate MR */
	    nt = (struct MailRecord *)SAmalloc(sizeof(struct MailRecord));
	    memcpy(nt, t, sizeof(struct MailRecord));
	    nt->next = t->next;
	    t->next = nt;
	    return nt;
	}
	t = t->next;
    }
    
    return NULL;			/* MR not found */
}


struct MailRecord *FindMRbyCode(short int icode) {
    struct MailRecord *t;
    
    /* find MR by code */
    for (t = sMR; t != NULL; t = t->next) if (t->code == icode) break;	/* MR found */
    
    return t;
}


void DelMR(struct MailRecord *iMR) {
    struct MailRecord *t, *ot = NULL;
    
    /* find previous MR */
    for (t = sMR; t != iMR; t = t->next) ot = t;

    /* update other MR affected by current */
    if (t->next != NULL) {		/* next MR exist */
	if (t == sMR) sMR = t->next;
	else ot->next = t->next;
    } else {				/* this is last MR */
	if (t == sMR) sMR = NULL;
	else ot->next = NULL;
    }
    
    free(t);
}


void DelMRs(unsigned int msgid) {
    struct MailRecord *t, *ot = NULL;
    
    for (t = sMR; t != NULL; t = t->next) {
	if (t->msgid == msgid) {
	    if (t == sMR) sMR = t->next;
	    else ot->next = t->next;
	    free(t);
	}
	ot = t;
    }
}


void DelAllMR(void) {
    struct MailRecord *t = sMR, *ot;
    
    while (t != NULL) {
	ot = t;
	t = t->next;
	free(ot);
    }
    
    sMR = NULL;
}

==> free-sa-1.6.2.tar.gz: free-sa-1.6.2/src/logformats/l_logformats.c <==
#include "global.h"


struct MailRecord *sMR = NULL;

struct LogFormat LogFormats[6] = { 
    {	"Squid 2.x native log format", 
	NULL,
	SquidReadRecord, 
	SquidReadDT,
	SquidReadFullDT,
	CommonReadAlign, {
	    1, 1, 1, 1, 1, 1, 1, /* TT reports */
	    1, 1, 1, 1, 1  	 /* TopSites, User, UserFull, UserGraph, UserDT */
	},
	1,
	PROXY },
	
    {	"CERN/NCSA Common Log Format (CLF)", 
	NULL,
	CLFReadRecord, 
	CLFReadDT,
	CLFReadFullDT,
	CommonReadAlign, {
	    1, 1, 1, 1, 1, 1, 1, /* TT reports */
	    0, 1, 1, 1, 1  	 /* TopSites, User, UserFull, UserGraph, UserDT */
	},
	1,
	WWW },
	
    {	"CERN/NCSA Combined Log Format", 
	NULL,
    	CombinedReadRecord, 
	CLFReadDT,
	CLFReadFullDT,
	CommonReadAlign, {
	    1, 1, 1, 1, 1, 1, 1, /* TT reports */
	    0, 1, 1, 1, 1     	 /* TopSites, User, UserFull, UserGraph, UserDT */
	},
	1,
	WWW },
	
    {	"Postfix 2.x over syslog log format",
	PFSLReadRecordM, 
	MailReadRecord, 
	SyslogReadDT,
	SyslogReadFullDT,
	CommonReadAlign, {
	    0, 0, 1, 1, 0, 0, 1, /* TT reports */
	    1, 1, 1, 1, 1	 /* TopSites, User, UserFull, UserGraph, UserDT */
	},
	1,
	MAIL },
	
    {	"Qmail over syslog log format",
	QMSLReadRecordM,
	MailReadRecord, 
	SyslogReadDT,
	SyslogReadFullDT,
	CommonReadAlign, {
	    0, 0, 1, 1, 0, 0, 1, /* TT reports */
	    1, 1, 1, 1, 1	 /* TopSites, User, UserFull, UserGraph, UserDT */
	},
	1,
	MAIL },
	
    {	"Communigate pro 5.x native log format",
	CGPReadRecordM,
	MailReadRecord, 
	SyslogReadDT,
	SyslogReadFullDT,
	CommonReadAlign, {
	    0, 0, 1, 1, 0, 0, 1, /* TT reports */
	    1, 1, 1, 1, 1	 /* TopSites, User, UserFull, UserGraph, UserDT */
	},
	1,
	MAIL }
};


void CommonReadAlign(FILE *iLog) {
    char t;
    
    while (fread(&t, 1L, 1L, iLog) == 1L) if (t == '\n') return;
    if (ferror(iLog)) SAexit("");
}


void CLFDTZ(time_t *iTime, char *idt) {
    struct tm ttm;
    
    if (strptime(idt, "%d/%b/%Y:%T %z", &ttm) == NULL) SAexit("Error converting date %s", idt);

    *iTime = SAmktime(&ttm);
}


char CLFReadDT(FILE *iLog, time_t *iTime) {
    char tdt[27];
    
    if (fscanf(iLog, "%*[^[]%*c%[^]]%*[^\n]%*c", tdt) != 1) return 0;
    
    CLFDTZ(iTime, tdt);
    
    return 1;
}


char CLFReadFullDT(FILE *iLog, time_t *iTime, char *iBuf) {
    char tdt[27];
    
    if (fgets(iBuf, RECORDSIZE, iLog) == NULL) return 0;
    if (sscanf(iBuf, "%*[^[]%*c%[^]]", tdt) != 1) return 0;
    
    CLFDTZ(iTime, tdt);
    
    return 1;
}


void SyslogDTZ(time_t *iTime, char *idt, short int year) {
    struct tm ttm;
    
    if (strptime(idt, "%b %d %T", &ttm) == NULL) SAexit("Error converting date %s", idt);

    ttm.tm_year = (int)(year - 1900);

    *iTime = SAmktime(&ttm);    
}


char SyslogReadDT(FILE *iLog, time_t *iTime) {
    char tdt[16];

    if (fscanf(iLog, "%15[^/] %*[^\n]%*c", tdt) != 1) return 0;
    
    SyslogDTZ(iTime, tdt, SYSLOGYEAR);
    
    return 1;
}


char SyslogReadFullDT(FILE *iLog, time_t *iTime, char *iBuf) {
    char tdt[16];

    if (fgets(iBuf, RECORDSIZE, iLog) == NULL) return 0;
    if (sscanf(iBuf, "%15[^/]", tdt) != 1) return 0;
    
    SyslogDTZ(iTime, tdt, SYSLOGYEAR);
    
    return 1;
}


char MailReadRecord(FILE *tLog, time_t *tTime, double *tSeconds, char *tIp, short int *tCode, double *tBytes, char *tMethod, char *tUrl, char *tName, char *tStat) {
    struct MailRecord *t = NULL, *nt;

    if ((t = READ_RECORDM(tLog)) == NULL) {
	for (t = sMR; t != NULL; t = nt) {
	    nt = t->next;
	    
	    if (t->status != 3) DelMR(t);
	    else break;
	}
    }
    
    if (t == NULL) return 0;
    
    strcpy(tIp, t->ip);
    strcpy(tUrl, t->url);
    strcpy(tName, t->name);
    *tBytes = t->bytes;
    *tTime = t->time;
    *tCode = t->code;
    *tStat = t->stat;
    *tSeconds = 0;
    tMethod[0] = 'G';

    if (--t->count) t->status = 2;			/* if multirecepient */
    else DelMR(t);					/* if last recepient */

    return 1;
}


struct MailRecord *FindNewMR(unsigned int msgid) {
    struct MailRecord *t = sMR, *ot = NULL;
    
    /* find MR by msgid */
    while(t != NULL) {
	if (t->msgid == msgid) return t; /* MR already exist */
	ot = t;
	t = t->next;
    }
    
    /* init new MR */
    t = (struct MailRecord *)SAmalloc(sizeof(struct MailRecord));
    t->msgid = msgid;
    t->status = -1;
    t->name[0] = '-';			/* handle empty from: <> */
    t->code = 0;
    t->count = 0;
    t->next = NULL;
    
    if (ot != NULL) ot->next = t;	/* init previous MR next */
    else sMR = t;			/* init start MR */
    
    return t;				/* MR not exist */
}


struct MailRecord *FindDupMR(unsigned int msgid) {
    struct MailRecord *t = sMR, *nt;
    
    /* find MR by msgid */
    while (t != NULL) {
	if (t->msgid == msgid) {	/* MR found */
	    if (!t->count) {		/* 1st recepient */
		t->count++;
		return t;
	    }
	    /* duplicate MR */
	    nt = (struct MailRecord *)SAmalloc(sizeof(struct MailRecord));
	    memcpy(nt, t, sizeof(struct MailRecord));
	    nt->next = t->next;
	    t->next = nt;
	    return nt;
	}
	t = t->next;
    }
    
    return NULL;			/* MR not found */
}


struct MailRecord *FindMRbyCode(short int icode) {
    struct MailRecord *t;
    
    /* find MR by code */
    for (t = sMR; t != NULL; t = t->next) if (t->code == icode) break;	/* MR found */
    
    return t;
}


void DelMR(struct MailRecord *iMR) {
    struct MailRecord *t, *ot = NULL;
    
    /* find previous MR */
    for (t = sMR; t != iMR; t = t->next) ot = t;

    /* update other MR affected by current */
    if (t->next != NULL) {		/* next MR exist */
	if (t == sMR) sMR = t->next;
	else ot->next = t->next;
    } else {				/* this is last MR */
	if (t == sMR) sMR = NULL;
	else ot->next = NULL;
    }
    
    free(t);
}


void DelMRs(unsigned int msgid) {
    struct MailRecord *t, *ot = NULL;
    
    for (t = sMR; t != NULL; t = t->next) {
	if (t->msgid == msgid) {
	    if (t == sMR) sMR = t->next;
	    else ot->next = t->next;
	    free(t);
	}
	ot = t;
    }
}


void DelAllMR(void) {
    struct MailRecord *t = sMR, *ot;
    
    while (t != NULL) {
	ot = t;
	t = t->next;
	free(ot);
    }
    
    sMR = NULL;
}
